function floyd_warshall (n, R)
{
	var i,j,k;

	for (k = 0 ; k < n ; k ++) {
		for (i = 0 ; i < n ; i ++) {
			for (j = 0 ; j < n ; j ++)
				R[i][j] = R[i][j] || (R[i][k] && R[k][j]);
		}
	}
	
	return R;
}


// in place, returns R
function rtc (n, R)
{
	// reflexive
	for (var i = 0 ; i < n ; ++i)
		R[i][i] = true;
	
	// 
	
	return floyd_warshall (n, R);
}

function list_to_mat (n,l)
{
	var R = [];
	for (var i = 0 ; i < n ; ++i) {
		R.push([]);
		for (var j = 0 ; j < n ; ++j)
			R[i][j] = false;
	}
	
	for (var k in l)
		R[l[k][0]-1][l[k][1]-1] = true;
	
	return R;
}

// total, 1 > 2=3 > 4
var R = [[1,2],[1,3],[2,4],[3,4],
         [2,3],[3,2]];

// not total, 1>4,6, 6>2,3>7, 4,7>5
var S = [[1,4],[1,6],[6,2],[6,3],[2,7],[3,7],[4,5],[7,5],
         [2,3],[3,2]];

// total, 3>1>4>2
var T = [[3,1],[1,4],[4,2]];

// not total
var U = [[1,2],[1,3],[2,4],[2,5],[2,6],[3,7],[4,10],[5,10],[6,11],[10,11],[6,8],[7,8],[7,9],[8,12],[9,12],[11,12],
         [4,5],[5,4],
         [8,9],[9,8]];

// [n,l] where n is the number of players and mat is a matrix.
var Tests = [[4,R],[7,S],[4,T],[12,U]];

// in Rhino
function verbose(s) { print(s); }

for (var t in Tests) {
	
	var n = Tests[t][0];
	var mat = rtc (n, list_to_mat(n, Tests[t][1]));
	var s;
	var i,j;
	
	verbose ("Running Test with Relation:____________________________");
	s = "";
	for (i = 0 ; i < n ; ++i) {
		s += "    ";
		for (j = 0 ; j < n ; ++j)
			s += mat[i][j] ? "1" : ".";
		s += "\n";
	}
	verbose (s);

	var p = new Preorder (n, function (i,j) { return mat[i][j] && !mat[j][i]; },
		function (i,j) { return mat[i][j] && mat[j][i]; });
	
	verbose ("Preorder at the beginning:_____________________________");
	verbose (p);
	
	p.isTotal();
	verbose ("Preorder after isTotal():______________________________");
	verbose (p);
	
	p.getEquivClasses();
	verbose ("Preorder after getEquivClasses():______________________");
	verbose (p);
	
	p.getDirectSuccessorsOf(0);
	verbose ("Preorder after getDirectSuccessorsOf(0):_______________");
	verbose (p);
	
	var diffCount = 0;
	var diff = [];
	/* Tests if the relations are equal: */
	for (i = 0 ; i < n ; ++i) {
		for (j = 0 ; j < n ; ++j) {
			if (p.isGreaterEqual(i,j) != mat[i][j]) {
				diffCount ++;
				diff.push(true);
			}
			else diff.push (false);
		}
	}
	
	if (diffCount > 0) {
		s = "";
		
		verbose ("Relations differ in " + diffCount + " positions (D means 'different'):");
		for (i = 0 ; i < n ; ++i) {
			s += "    ";
			for (j = 0 ; j < n ; ++j)
				s += diff[i*n+j] ? "D" : ".";
			s += "\n";
		}
		verbose (s);
	}
	else {
		verbose ("Relations are equal: GOOD");
	}
	verbose ("==============================================\n");
}